package modele.info1.ships;

import info1.ships.INavyFleet;
import info1.ships.IShip;
import info1.ships.ShipCategory;


import java.util.*;

/**
 * Classe définissant une flotte de navires
 */

public class NavyFleet implements INavyFleet {

    ArrayList<IShip> fleet;

    /**
     * NB : LA SIGNATURE DU CONSTRUCTEUR DOIT ETRE RESPECTEE
     *
     * Construit une nouvelle flotte
     */
    public NavyFleet() {
        fleet = new ArrayList<IShip>();
    }

    @Override
    public int remainingSize() {
        int tot = 20;
        for (IShip ship : fleet){
            tot = tot - ship.getSize();
        }
        return tot;
    }

    @Override
    public boolean isComplete() {
        return remainingSize() == 0;
    }


    @Override
    public int add(IShip ship) {
        if(fleet.contains(ship))
            return -1;
        else if(remainingSize() - ship.getSize() < 0)
            return -2;
        else if (isOnPlace(ship))
            return -3;
        else {
            fleet.add(ship);
            return 0;
        }
    }

    private boolean isOnPlace(IShip ship){
        for (IShip shipp : fleet){
            for (int i = 0; i < shipp.getCoords().size(); i++) {
                if (ship.getCoords().contains(shipp.getCoords().get(i)))
                    return true;
            }
        }
        return false;
    }


    @Override
    public List<IShip> getShips() {
        List<IShip> tmp = fleet;
        Collections.sort(tmp);
        return tmp;
    }

    @Override
    public Set<IShip> getShips(ShipCategory shipCategory) {
        Set<IShip> tmp = new LinkedHashSet<>();
        for (IShip ship : fleet)
            if(ship.getCategory()==shipCategory)
                tmp.add(ship);

        return tmp;
    }

    @Override
    public boolean isBelgianConfiguration() {
        // TO DO verifier 1 cuirassÃ©, 2 croiseurs, 3 torpilleurs, 4 sous-marins
        return getShips(ShipCategory.BATTLESHIP).size() == 1
                && getShips(ShipCategory.CRUISER).size() == 2
                && getShips(ShipCategory.DESTROYER).size() == 3
                && getShips(ShipCategory.SUBMARINE).size() == 4;
    }

    @Override
    public boolean isFrenchConfiguration() {
        // TO DO verifier: 1 porte-avion, 1 cuirassÃ©, 2 croiseurs, 2 torpilleurs, 1 sous-marin
        return getShips(ShipCategory.AIRCRAFT_CARRIER).size() == 1
                && getShips(ShipCategory.BATTLESHIP).size() == 1
                && getShips(ShipCategory.CRUISER).size() == 2
                && getShips(ShipCategory.DESTROYER).size() == 2
                && getShips(ShipCategory.SUBMARINE).size() == 1;
    }


    @Override
    public String toString() {
        return "NavyFleet{" +
                "fleet=" + fleet +
                '}';
    }
}
